@@ -3,7 +3,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = "0.12.15"
+VERSION = "0.12.15.1"
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = "local"
@@ -67,6 +67,9 @@ CONFIG_UPDATE_PATH = "/?_config_"
 # environment variable name to tag local test runs
 ENV_INTERNAL_TEST_RUN = "LOCALSTACK_INTERNAL_TEST_RUN"
 
+# environment variable that flags whether pro was activated. do not use for security purposes!
+ENV_PRO_ACTIVATED = "PRO_ACTIVATED"
+
 # content types
 APPLICATION_AMZ_JSON_1_0 = "application/x-amz-json-1.0"
 APPLICATION_AMZ_JSON_1_1 = "application/x-amz-json-1.1"
