@@ -1,14 +1,17 @@
-import os
 import json
-from localstack.utils import common
+import os
+
 from localstack.constants import INSTALL_DIR_INFRA
+from localstack.utils import common
 
 # URL to "cfn-response" module which is required in some CF Lambdas
-CFN_RESPONSE_MODULE_URL = 'https://raw.githubusercontent.com/LukeMizuhashi/cfn-response/master/index.js'
+CFN_RESPONSE_MODULE_URL = (
+    "https://raw.githubusercontent.com/LukeMizuhashi/cfn-response/master/index.js"
+)
 
 # placeholders
-PLACEHOLDER_RESOURCE_NAME = '__resource_name__'
-PLACEHOLDER_AWS_NO_VALUE = '__aws_no_value__'
+PLACEHOLDER_RESOURCE_NAME = "__resource_name__"
+PLACEHOLDER_AWS_NO_VALUE = "__aws_no_value__"
 
 
 def dump_json_params(param_func=None, *param_names):
@@ -21,6 +24,7 @@ def dump_json_params(param_func=None, *param_names):
                 param_value = common.json_safe(result[name])
                 result[name] = json.dumps(param_value)
         return result
+
     return replace
 
 
@@ -28,14 +32,16 @@ def param_defaults(param_func, defaults):
     def replace(params, **kwargs):
         result = param_func(params, **kwargs)
         for key, value in defaults.items():
-            if result.get(key) in ['', None]:
+            if result.get(key) in ["", None]:
                 result[key] = value
         return result
+
     return replace
 
 
 def remove_none_values(params):
-    """ Remove None values recursively in the given object. """
+    """Remove None values recursively in the given object."""
+
     def remove_nones(o, **kwargs):
         if isinstance(o, dict):
             for k, v in dict(o).items():
@@ -45,11 +51,12 @@ def remove_none_values(params):
             common.run_safe(o.remove, None)
             common.run_safe(o.remove, PLACEHOLDER_AWS_NO_VALUE)
         return o
+
     result = common.recurse_object(params, remove_nones)
     return result
 
 
-def params_list_to_dict(param_name, key_attr_name='Key', value_attr_name='Value'):
+def params_list_to_dict(param_name, key_attr_name="Key", value_attr_name="Value"):
     def do_replace(params, **kwargs):
         result = {}
         for entry in params.get(param_name, []):
@@ -57,6 +64,7 @@ def params_list_to_dict(param_name, key_attr_name='Key', value_attr_name='Value'
             value = entry[value_attr_name]
             result[key] = value
         return result
+
     return do_replace
 
 
@@ -65,14 +73,16 @@ def lambda_keys_to_lower(key=None):
 
 
 def merge_parameters(func1, func2):
-    return lambda params, **kwargs: common.merge_dicts(func1(params, **kwargs), func2(params, **kwargs))
+    return lambda params, **kwargs: common.merge_dicts(
+        func1(params, **kwargs), func2(params, **kwargs)
+    )
 
 
 def str_or_none(o):
     return o if o is None else json.dumps(o) if isinstance(o, (dict, list)) else str(o)
 
 
-def params_dict_to_list(param_name, key_attr_name='Key', value_attr_name='Value', wrapper=None):
+def params_dict_to_list(param_name, key_attr_name="Key", value_attr_name="Value", wrapper=None):
     def do_replace(params, **kwargs):
         result = []
         for key, value in params.get(param_name, {}).items():
@@ -80,6 +90,7 @@ def params_dict_to_list(param_name, key_attr_name='Key', value_attr_name='Value'
         if wrapper:
             result = {wrapper: result}
         return result
+
     return do_replace
 
 
@@ -90,6 +101,7 @@ def params_select_attributes(*attrs):
             if params.get(attr) is not None:
                 result[attr] = str_or_none(params.get(attr))
         return result
+
     return do_select
 
 
@@ -99,11 +111,12 @@ def param_json_to_str(name):
         if result:
             result = json.dumps(result)
         return result
+
     return _convert
 
 
 def get_cfn_response_mod_file():
-    cfn_response_tmp_file = os.path.join(INSTALL_DIR_INFRA, 'lambda.cfn-response.js')
+    cfn_response_tmp_file = os.path.join(INSTALL_DIR_INFRA, "lambda.cfn-response.js")
     if not os.path.exists(cfn_response_tmp_file):
         common.download(CFN_RESPONSE_MODULE_URL, cfn_response_tmp_file)
     return cfn_response_tmp_file
