@@ -11,8 +11,7 @@ from moto.ses.responses import ses_backend
 
 from localstack import config
 from localstack.services.infra import start_moto_server
-from localstack.utils.common import mkdir
-from localstack.utils.common import to_str, timestamp_millis
+from localstack.utils.common import mkdir, timestamp_millis, to_str
 
 LOGGER = logging.getLogger(__name__)
 
@@ -47,30 +46,29 @@ GET_IDENTITY_VERIFICATION_ATTRIBUTES_RESPONSE = """<GetIdentityVerificationAttri
 
 def apply_patches():
     def get_source_from_raw(raw_data):
-        entities = raw_data.split('\n')
+        entities = raw_data.split("\n")
         for entity in entities:
-            if 'From: ' in entity:
-                return entity.replace('From: ', '').strip()
+            if "From: " in entity:
+                return entity.replace("From: ", "").strip()
 
         return None
 
     email_responses_send_raw_email_orig = email_responses.send_raw_email
 
     def email_responses_send_raw_email(self):
-        (source,) = self.querystring.get('Source', [''])
+        (source,) = self.querystring.get("Source", [""])
         if source.strip():
             return email_responses_send_raw_email_orig(self)
 
-        raw_data = to_str(base64.b64decode(
-            self.querystring.get('RawMessage.Data')[0]))
+        raw_data = to_str(base64.b64decode(self.querystring.get("RawMessage.Data")[0]))
 
-        LOGGER.debug('Raw email:\n%s' % raw_data)
+        LOGGER.debug("Raw email:\n%s" % raw_data)
 
         source = get_source_from_raw(raw_data)
         if not source:
-            raise MessageRejectedError('Source not specified')
+            raise MessageRejectedError("Source not specified")
 
-        self.querystring['Source'] = [source]
+        self.querystring["Source"] = [source]
         return email_responses_send_raw_email_orig(self)
 
     email_responses.send_raw_email = email_responses_send_raw_email
@@ -78,26 +76,27 @@ def apply_patches():
     email_responses_send_email_orig = email_responses.send_email
 
     def email_responses_send_email(self):
-        bodydatakey = 'Message.Body.Text.Data'
-        if 'Message.Body.Html.Data' in self.querystring:
-            bodydatakey = 'Message.Body.Html.Data'
+        bodydatakey = "Message.Body.Text.Data"
+        if "Message.Body.Html.Data" in self.querystring:
+            bodydatakey = "Message.Body.Html.Data"
 
         body = self.querystring.get(bodydatakey)[0]
-        source = self.querystring.get('Source')[0]
-        subject = self.querystring.get('Message.Subject.Data')[0]
-        destinations = {'ToAddresses': [],
-                        'CcAddresses': [], 'BccAddresses': []}
+        source = self.querystring.get("Source")[0]
+        subject = self.querystring.get("Message.Subject.Data")[0]
+        destinations = {"ToAddresses": [], "CcAddresses": [], "BccAddresses": []}
         for dest_type in destinations:
             # consume up to 51 to allow exception
             for i in range(1, 52):
-                field = 'Destination.%s.member.%s' % (dest_type, i)
+                field = "Destination.%s.member.%s" % (dest_type, i)
                 address = self.querystring.get(field)
                 if address is None:
                     break
                 destinations[dest_type].append(address[0])
 
-        LOGGER.debug('Raw email\nFrom: %s\nTo: %s\nSubject: %s\nBody:\n%s'
-                     % (source, destinations, subject, body))
+        LOGGER.debug(
+            "Raw email\nFrom: %s\nTo: %s\nSubject: %s\nBody:\n%s"
+            % (source, destinations, subject, body)
+        )
 
         return email_responses_send_email_orig(self)
 
@@ -108,15 +107,15 @@ def apply_patches():
     def list_templates(self):
         email_templates = ses_backend.list_templates()
         for template in email_templates:
-            if isinstance(template['Timestamp'], (date, datetime)):
+            if isinstance(template["Timestamp"], (date, datetime)):
                 # Hack to change the last digits to Java SDKv2 compatible format
-                template['Timestamp'] = timestamp_millis(template['Timestamp'])
+                template["Timestamp"] = timestamp_millis(template["Timestamp"])
         return email_responses_list_templates_orig(self)
 
     email_responses.list_templates = list_templates
 
     def delete_template(self):
-        template_name = self._get_param('TemplateName')
+        template_name = self._get_param("TemplateName")
         templates = ses_backend.templates
         if template_name in templates:
             del templates[template_name]
@@ -127,8 +126,11 @@ def apply_patches():
     email_responses.delete_template = delete_template
 
     def get_identity_verification_attributes(self):
-        resources = [self.querystring[identity][0] for identity
-                     in self.querystring.keys() if 'Identities.member' in identity]
+        resources = [
+            self.querystring[identity][0]
+            for identity in self.querystring.keys()
+            if "Identities.member" in identity
+        ]
         template = self.response_template(GET_IDENTITY_VERIFICATION_ATTRIBUTES_RESPONSE)
         return template.render(resources=resources)
 
@@ -139,17 +141,21 @@ def apply_patches():
     def send_email_save_contents(self, source, subject, body, destinations, region):
         message = backend_send_email_orig(self, source, subject, body, destinations, region)
 
-        ses_dir = os.path.join(config.DATA_DIR or config.TMP_FOLDER, 'ses')
+        ses_dir = os.path.join(config.DATA_DIR or config.TMP_FOLDER, "ses")
         mkdir(ses_dir)
 
-        with (open(os.path.join(ses_dir, message.id + '.json'), 'w')) as f:
-            f.write(json.dumps({
-                'Source': source,
-                'Subject': subject,
-                'Body': body,
-                'Destinations': destinations,
-                'Region': region
-            }))
+        with (open(os.path.join(ses_dir, message.id + ".json"), "w")) as f:
+            f.write(
+                json.dumps(
+                    {
+                        "Source": source,
+                        "Subject": subject,
+                        "Body": body,
+                        "Destinations": destinations,
+                        "Region": region,
+                    }
+                )
+            )
         return message
 
     SESBackend.send_email = send_email_save_contents
@@ -159,10 +165,10 @@ def start_ses(port=None, backend_port=None, asynchronous=None, update_listener=N
     port = port or config.PORT_SES
     apply_patches()
     return start_moto_server(
-        key='ses',
-        name='SES',
+        key="ses",
+        name="SES",
         port=port,
         backend_port=backend_port,
         asynchronous=asynchronous,
-        update_listener=update_listener
+        update_listener=update_listener,
     )
