@@ -14,6 +14,7 @@ import traceback
 import warnings
 from concurrent.futures._base import Future
 from datetime import datetime
+from typing import List, Union
 
 import pip as pip_mod
 import six
@@ -419,12 +420,11 @@ def validate_localstack_config(name):
     warns = []
 
     # validating docker-compose file
-    cmd = "docker-compose -f '%s' config" % (compose_file_name)
+    cmd = ["docker-compose", "-f", compose_file_name, "config"]
     try:
-        run(cmd)
+        run(cmd, shell=False)
     except Exception as e:
         LOG.warning("Looks like the docker-compose file is not valid: %s" % e)
-        return False
 
     # validating docker-compose variable
     import yaml  # keep import here to avoid issues in test Lambdas
@@ -454,7 +454,7 @@ def validate_localstack_config(name):
     network_mode = ls_service_details.get("network_mode")
     image_name = ls_service_details.get("image")
     container_name = ls_service_details.get("container_name") or ""
-    docker_ports = (port.split(":")[0] for port in ls_service_details.get("ports", []))
+    docker_ports = (port.split(":")[-2] for port in ls_service_details.get("ports", []))
     docker_env = dict(
         (env.split("=")[0], env.split("=")[1]) for env in ls_service_details.get("environment", {})
     )
@@ -528,7 +528,7 @@ class PortMappings(object):
         self.bind_host = bind_host if bind_host else ""
         self.mappings = {}
 
-    def add(self, port, mapped=None):
+    def add(self, port, mapped=None, protocol="tcp"):
         mapped = mapped or port
         if isinstance(port, list):
             for i in range(port[1] - port[0] + 1):
@@ -546,18 +546,31 @@ class PortMappings(object):
             self.expand_range(port, from_range)
             self.expand_range(mapped, to_range)
             return
-        self.mappings[self.HashableList([port, port])] = [mapped, mapped]
+        protocol = str(protocol or "tcp").lower()
+        self.mappings[self.HashableList([port, port, protocol])] = [mapped, mapped]
 
-    def to_str(self):
+    def to_str(self) -> str:  # TODO test (and/or remove?)
         bind_address = f"{self.bind_host}:" if self.bind_host else ""
 
         def entry(k, v):
+            protocol = "/%s" % k[2] if k[2] != "tcp" else ""
             if k[0] == k[1] and v[0] == v[1]:
-                return "-p %s%s:%s" % (bind_address, k[0], v[0])
-            return "-p %s%s-%s:%s-%s" % (bind_address, k[0], k[1], v[0], v[1])
+                return "-p %s%s:%s%s" % (bind_address, k[0], v[0], protocol)
+            return "-p %s%s-%s:%s-%s%s" % (bind_address, k[0], k[1], v[0], v[1], protocol)
 
         return " ".join([entry(k, v) for k, v in self.mappings.items()])
 
+    def to_list(self) -> List[str]:  # TODO test
+        bind_address = f"{self.bind_host}:" if self.bind_host else ""
+
+        def entry(k, v):
+            protocol = "/%s" % k[2] if k[2] != "tcp" else ""
+            if k[0] == k[1] and v[0] == v[1]:
+                return ["-p", f"{bind_address}{k[0]}:{v[0]}{protocol}"]
+            return ["-p", f"{bind_address}{k[0]}-{k[1]}:{v[0]}-{v[1]}{protocol}"]
+
+        return [item for k, v in self.mappings.items() for item in entry(k, v)]
+
     def contains(self, port):
         for from_range, to_range in self.mappings.items():
             if self.in_range(port, from_range):
@@ -798,7 +811,7 @@ class FuncThread(threading.Thread):
 
 
 def run(
-    cmd,
+    cmd: Union[str, List[str]],
     print_error=True,
     asynchronous=False,
     stdin=False,
@@ -808,7 +821,9 @@ def run(
     inherit_cwd=False,
     inherit_env=True,
     tty=False,
+    shell=True,
 ):
+    LOG.debug("Executing command: %s", cmd)
     env_dict = os.environ.copy() if inherit_env else {}
     if env_vars:
         env_dict.update(env_vars)
@@ -823,14 +838,9 @@ def run(
         if not asynchronous:
             if stdin:
                 return subprocess.check_output(
-                    cmd,
-                    shell=True,
-                    stderr=stderr,
-                    env=env_dict,
-                    stdin=subprocess.PIPE,
-                    cwd=cwd,
+                    cmd, shell=shell, stderr=stderr, env=env_dict, stdin=subprocess.PIPE, cwd=cwd
                 )
-            output = subprocess.check_output(cmd, shell=True, stderr=stderr, env=env_dict, cwd=cwd)
+            output = subprocess.check_output(cmd, shell=shell, stderr=stderr, env=env_dict, cwd=cwd)
             return output.decode(config.DEFAULT_ENCODING)
 
         stdin_arg = subprocess.PIPE if stdin else None
@@ -847,10 +857,10 @@ def run(
         # start the actual sub process
         kwargs = {}
         if is_linux() or is_mac_os():
-            kwargs["preexec_fn"] = os.setsid
+            kwargs["start_new_session"] = True
         process = subprocess.Popen(
             cmd,
-            shell=True,
+            shell=shell,
             stdin=stdin_arg,
             bufsize=-1,
             stderr=stderr_arg,
