@@ -25,6 +25,7 @@ from contextlib import closing
 from datetime import date, datetime, timezone
 from multiprocessing.dummy import Pool
 from queue import Queue
+from typing import Callable, List, Union
 from urllib.parse import parse_qs, urlparse
 
 import dns.resolver
@@ -1117,12 +1118,40 @@ def extract_jsonpath(value, path):
     return result
 
 
-def assign_to_path(target, path, value):
-    path = path.split(".")
-    for i in range(len(path) - 1):
-        target_new = target[path[i]] = target.get(path[i], {})
+def assign_to_path(target, path: str, value, delimiter: str = "."):
+    parts = path.strip(delimiter).split(delimiter)
+    path_to_parent = delimiter.join(parts[:-1])
+    parent = extract_from_jsonpointer_path(target, path_to_parent, auto_create=True)
+    if not isinstance(parent, dict):
+        LOG.debug(
+            'Unable to find parent (type %s) for path "%s" in object: %s'
+            % (type(parent), path, target)
+        )
+        return
+    path_end = int(parts[-1]) if is_number(parts[-1]) else parts[-1]
+    parent[path_end] = value
+    return target
+
+
+def extract_from_jsonpointer_path(target, path: str, delimiter: str = "/", auto_create=False):
+    parts = path.strip(delimiter).split(delimiter)
+    for part in parts:
+        path_part = int(part) if is_number(part) else part
+        if isinstance(target, list) and not is_number(path_part):
+            if path_part == "-":
+                # special case where path is like /path/to/list/- where "/-" means "append to list"
+                continue
+            LOG.warning(
+                'Attempting to extract non-int index "%s" from list: %s' % (path_part, target)
+            )
+            return None
+        target_new = target[path_part] if isinstance(target, list) else target.get(path_part)
+        if target_new is None:
+            if not auto_create:
+                return
+            target[path_part] = target_new = {}
         target = target_new
-    target[path[-1]] = value
+    return target
 
 
 def save_file(file, content, append=False):
@@ -1178,6 +1207,14 @@ def to_bytes(obj, encoding=DEFAULT_ENCODING, errors="strict"):
     return obj.encode(encoding, errors) if isinstance(obj, six.text_type) else obj
 
 
+def str_to_bool(value):
+    """Return the boolean value of the given string, or the verbatim value if it is not a string"""
+    true_strings = ["true", "True"]
+    if isinstance(value, str):
+        return value in true_strings
+    return value
+
+
 def str_insert(string, index, content):
     """Insert a substring into an existing string at a certain index."""
     return "%s%s%s" % (string[:index], content, string[index:])
@@ -1545,12 +1582,9 @@ def run_for_max_seconds(max_secs, _function, *args, **kwargs):
         time.sleep(0.5)
 
 
-def run(cmd, cache_duration_secs=0, **kwargs):
-    def do_run(cmd):
-        return bootstrap.run(cmd, **kwargs)
-
+def do_run(cmd: str, run_cmd: Callable, cache_duration_secs: int):
     if cache_duration_secs <= 0:
-        return do_run(cmd)
+        return run_cmd()
 
     hashcode = md5(cmd)
     cache_file = CACHE_FILE_PATTERN.replace("*", hashcode)
@@ -1562,13 +1596,27 @@ def run(cmd, cache_duration_secs=0, **kwargs):
         if mod_time > (time_now - cache_duration_secs):
             with open(cache_file) as fd:
                 return fd.read()
-    result = do_run(cmd)
+    result = run_cmd()
     with open(cache_file, "w+") as fd:
         fd.write(result)
     clean_cache()
     return result
 
 
+def run(cmd, cache_duration_secs=0, **kwargs):
+    def run_cmd():
+        return bootstrap.run(cmd, **kwargs)
+
+    return do_run(cmd, run_cmd, cache_duration_secs)
+
+
+def safe_run(cmd: List[str], cache_duration_secs=0, **kwargs) -> Union[str, subprocess.Popen]:
+    def run_cmd():
+        return bootstrap.run(cmd, shell=False, **kwargs)
+
+    return do_run(" ".join(cmd), run_cmd, cache_duration_secs)
+
+
 def clone(item):
     return json.loads(json.dumps(item))
 
